/****************************************************************************
** $Id: qabstractlayout.i,v 1.1.1.1 2002/12/22 14:19:59 taka-hr Exp $
**
** Definition of the abstract layout base class
**
** Created : 960416
**
** Copyright (C) 1992-2000 Troll Tech AS.  All rights reserved.
**
** This file is part of the Qt GUI Toolkit.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Troll Tech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** Licensees holding valid Qt Professional Edition licenses may use this
** file in accordance with the Qt Professional Edition License Agreement
** provided with the Qt Professional Edition.
**
** See http://www.trolltech.com/pricing.html or email sales@trolltech.com for
** information about the Professional Edition licensing, or see
** http://www.trolltech.com/qpl/ for QPL licensing information.
**
*****************************************************************************/

//#ifndef QABSTRACTLAYOUT_H
//#define QABSTRACTLAYOUT_H

//#ifndef QT_H
//#include "qobject.h"
//#include "qsizepolicy.h"
//#include "qwidget.h"
//#endif // QT_H

class QMenuBar;
class QWidget;
struct QLayoutData;
class QLayoutItem;
class QLayout;
class QSpacerItem;
class QDomElement;
class QConfigureLayoutEvent;

class QGLayoutIterator : public QShared
{
public:
		virtual ~QGLayoutIterator();
		virtual QLayoutItem *next() = 0;
		virtual QLayoutItem *current() = 0;
		virtual QLayoutItem *takeCurrent() = 0;
};

class QLayoutIterator
{
public:
//    QLayoutIterator( QGLayoutIterator *i ) :it(i) {}
		QLayoutIterator( const QLayoutIterator &i ); // :it( i.it )
//    { if ( it ) it->ref(); }
		~QLayoutIterator() { if ( it && it->deref() ) delete it; }
//    QLayoutIterator &operator=( const QLayoutIterator &i )
//    {
//	if ( i.it ) i.it->ref();
//	if ( it && it->deref() ) delete it;
//	it = i.it;
//	return *this;
//    }
//    QLayoutItem *operator++() { return it ? it->next() : 0; }
		QLayoutItem *current() { return it ? it->current() : 0; }
		QLayoutItem *takeCurrent() { return it ? it->takeCurrent() : 0; }
		void deleteCurrent();
private:
		QGLayoutIterator *it;
};


class QLayoutItem
{
public:
//    QLayoutItem( int alignment = 0 ) :align(alignment) {}
		virtual ~QLayoutItem();
		virtual QSize sizeHint() const = 0;
		virtual QSize minimumSize() const = 0;
		virtual QSize maximumSize() const = 0;
		virtual QSizePolicy::ExpandData expanding() const =0;
		virtual void setGeometry( const QRect& ) = 0;
		virtual QRect geometry() const = 0;
		virtual bool isEmpty() const = 0;
		virtual bool hasHeightForWidth() const;
		virtual int heightForWidth( int ) const;
		virtual void invalidate();

		virtual QWidget *widget();
		virtual QLayoutIterator iterator();
		virtual QLayout *layout();
		virtual QSpacerItem *spacerItem();

		int alignment() const { return align; }
		virtual void setAlignment( int a );
protected:
		int align;
};


class QSpacerItem : public QLayoutItem
{
 public:
		QSpacerItem( int w, int h,
		 QSizePolicy::SizeType hData=QSizePolicy::Minimum,
		 QSizePolicy::SizeType vData= QSizePolicy::Minimum )
	:width(w), height(h), sizeP(hData, vData )
	{}
		void changeSize( int w, int h,
		QSizePolicy::SizeType hData=QSizePolicy::Minimum,
		QSizePolicy::SizeType vData=QSizePolicy::Minimum );
		QSize sizeHint() const ;
		QSize minimumSize() const ;
		QSize maximumSize() const ;
		QSizePolicy::ExpandData expanding() const;
		bool isEmpty() const;
		void setGeometry( const QRect& );
		QRect geometry() const;
		QSpacerItem *spacerItem();
private:
		int width, height;
		QSizePolicy sizeP;
		QRect rect;
};


class QWidgetItem : public QLayoutItem
{
public:
		QWidgetItem( QWidget *w ) : wid(w) {}
		QSize sizeHint() const ;
		QSize minimumSize() const ;
		QSize maximumSize() const ;
		QSizePolicy::ExpandData expanding() const;
		bool isEmpty() const;
		void setGeometry( const QRect& ) ;
		QRect geometry() const;
		//void invalidate();
		virtual QWidget *widget();

		bool hasHeightForWidth() const;
		int heightForWidth( int ) const;

private:
		//QSize cachedSizeHint;
		QWidget *wid;
};


class QLayout : public QObject //, public QLayoutItem for swig 1.3a bug
{
/*   
		Q_OBJECT
		Q_ENUMS( ResizeMode )
		Q_PROPERTY( int margin READ margin WRITE setMargin )
		Q_PROPERTY( int spacing READ spacing WRITE setSpacing )
		Q_PROPERTY( ResizeMode resizeMode READ resizeMode WRITE setResizeMode )
*/    
public:
/*
%extend {
		QLayout( VALUE vparent, 
								 VALUE varg1 = 0, VALUE varg2 = 0, VALUE varg3 = 0) {
				QWidget* wparent = 0;
				QLayout* lparent = 0;
				int margin = 0;
				int space = -1;
				char* name = 0;
				if(rb_obj_is_kind_of(vparent,cQWidget) || TYPE(vparent) == T_NIL) {
					 wparent = (QWidget*)Get_QWidget(vparent);
					 if(varg1 != 0) margin = NUM2INT(varg1);
					 if(varg2 != 0) space = NUM2INT(varg2);
					 if(varg3 != 0) name = STR2CSTR(varg3);
					 return new QLayout(wparent,margin,space,name);

				} else if(rb_obj_is_kind_of(vparent,cQLayout) || TYPE(vparent) == T_NIL) {
					 lparent = (QLayout*)Get_QLayout(vparent);
					 if(varg1 != 0) space = NUM2INT(varg1);
					 if(varg2 != 0) name = STR2CSTR(varg2);
					 return new QLayout(lparent,space,name);
					 
				} else {
					 rb_raise(rb_eArgError, "Type error in QLayout constructor");
				}

		 }
}
*/
//    QLayout( QWidget *parent, int margin=0, int space=-1,
//	     const char *name=0 );
//    QLayout( QLayout *parentLayout, int space=-1, const char *name=0 );
//    QLayout( int space=-1, const char *name=0 );

		~QLayout();
		
		int margin() const { return outsideBorder; }
		int spacing() const { return insideSpacing; }

		virtual void setMargin( int );
		virtual void setSpacing( int );

		enum { unlimited = QWIDGETSIZE_MAX };
//#if 1 //OBSOLETE
		int defaultBorder() const { return insideSpacing; }
//    void freeze( int w, int h );
		//RENAME %name(freezeFixed) void freeze() { setResizeMode( Fixed ); }
		%extend {
			 void freeze(VALUE varg0=Qnil,VALUE varg1=Qnil) {
					if(varg0==Qnil) {
						 self->freeze();
					} else {
						 int w,h;
						 w = NUM2INT(varg0);
						 h = NUM2INT(varg1);
						 self->freeze(w,h);
					}
			 }
		}
//#endif

		enum ResizeMode { FreeResize, Minimum, Fixed };
		void setResizeMode( ResizeMode );
		ResizeMode resizeMode() const;

		virtual void  setMenuBar( QMenuBar *w );

		QWidget *mainWidget();
		QMenuBar *menuBar() const { return menubar; }
		bool isTopLevel() const { return topLevel; }

		virtual void setAutoAdd( bool );
		bool autoAdd() const { return autoNewChild; }

		void invalidate();
		QRect geometry() const;
		bool activate();

		void add( QWidget *w ) { addItem( new QWidgetItem( w ) ); }
		virtual void addItem ( QLayoutItem * ) = 0;

		QSizePolicy::ExpandData expanding() const;
		QSize minimumSize() const;
		QSize maximumSize() const;
//    void setGeometry( const QRect& )=0;
//    QLayoutIterator iterator()=0;
		bool isEmpty() const;

		int totalHeightForWidth( int w ) const;
		QSize totalMinimumSize() const;
		QSize totalMaximumSize() const;
		QSize totalSizeHint() const;
		QLayout *layout();

		bool supportsMargin() const { return marginImpl; }
/*
protected:
		bool  eventFilter( QObject *, QEvent * );
		void addChildLayout( QLayout *l );
		void deleteAllItems();

		void setSupportsMargin( bool );
		QRect alignmentRect( const QRect& ) const;
private:
		void setWidgetLayout( QWidget *, QLayout * );
		void init();
		int insideSpacing;
		int outsideBorder;
		uint topLevel : 1;
		uint autoMinimum : 1;
		uint autoNewChild : 1;
		uint frozen : 1;
		uint activated : 1;
		uint marginImpl : 1;
		QRect rect;
		QLayoutData *extraData;
		QMenuBar *menubar;
private:	// Disabled copy constructor and operator=
#if defined(Q_DISABLE_COPY)
		QLayout( const QLayout & );
		QLayout &operator=( const QLayout & );
#endif
*/
//QLayoutItem methods
		virtual QSize sizeHint() const = 0;
//    virtual QSize minimumSize() const = 0;
//    virtual QSize maximumSize() const = 0;
//    virtual QSizePolicy::ExpandData expanding() const =0;
		virtual void setGeometry( const QRect& ) = 0;
//    virtual QRect geometry() const = 0;
//    virtual bool isEmpty() const = 0;
		virtual bool hasHeightForWidth() const;
		virtual int heightForWidth( int ) const;
//    virtual void invalidate();

		virtual QWidget *widget();
		virtual QLayoutIterator iterator();
//    virtual QLayout *layout();
		virtual QSpacerItem *spacerItem();

		int alignment() const { return align; }
		virtual void setAlignment( int a );
};


//#endif
